/*

    qmailalizer - a qmail-send log analysis program

    Copyright (C) 2001  Pichler Wolfgang  (wpichler@dialog-gruppe.at)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

    This software uses the gd graphics library, which is copyright by
    Quest Protein Database Center, Cold Spring Harbor Labs.  Please
    see the documentation supplied with the library for additional
    information and license terms, or visit www.boutell.com/gd/ for the
    most recent version of the library and supporting documentation.


*/

#include "xmlinput.h"
#include <time.h>
#include <string.h>

XMLInput::XMLInput (Config *config_p,MHandler *mhandler_p,Results *result_p)
{
  config = config_p;
  mhandler = mhandler_p;
  results = result_p;
  lWurzel = NULL;
}

XMLInput::~XMLInput ()
{

}

// Arbeitet die einzelnen Schritte ab
int XMLInput::LoadResults ()
{
  if ((result = InitXML ()) != NULL) {
    InitNodes ();
    LoadFiles ();
    LoadDomains ();
    LoadMessages ();
    FreeXML ();
    return 0;
  }
  else
    return -1;
}

// Ldt die xml Datei - und parst diese
xmlDocPtr XMLInput::InitXML ()
{ xmlNodePtr cur;
  xmlDocPtr doc;
  
  doc = xmlParseFile (config->xmlfile);
  if (doc == NULL) return NULL;
  cur = xmlDocGetRootElement(doc);
  if (cur == NULL) { xmlFreeDoc (doc); return NULL; }
  if (xmlStrcmp(cur->name,(xmlChar *)"RESULTS")) { xmlFreeDoc(doc); return NULL; }
  // Wenn wir hier sind dann haben wir das richtige File
  // Also den Pointer darauf zurckliefern
  printf ("Stats Generated sucessfully.\n");
  return doc;
}

// gibt die Datei wieder frei
void XMLInput::FreeXML ()
{
  xmlFreeDoc (result);
  xmlCleanupParser ();
}

// Erstellt die Bezge auf die 3 Anfangs Nodes
void XMLInput::InitNodes ()
{ xmlNodePtr cur;
  
  cur = xmlDocGetRootElement(result);
  cur = cur->xmlChildrenNode;
  while (cur != NULL) {
    if (!xmlStrcmp(cur->name,(xmlChar *)"FILES")) {
      NFiles = cur->xmlChildrenNode;
    }
    else if (!xmlStrcmp(cur->name,(xmlChar *)"OPENMESSAGES")) {
      NMessages = cur->xmlChildrenNode;
    }
    else if (!xmlStrcmp(cur->name,(xmlChar *)"DOMAINS")) {
      NDomains = cur->xmlChildrenNode;
    }
    cur = cur->next;
  }
}

// Ldt die Domain Daten
void XMLInput::LoadDomains ()
{ xmlNodePtr cur;
  xmlNodePtr member;
  xmlNodePtr month;
  struct tm *start,*end;
  Domains *aktdomain;
  unsigned long usage[12];
  unsigned long bytes[12];
  int year;
  int i;
  
  for (i=0;i<12;i++) {
    usage[i] = 0;
    bytes[i] = 0;
  }
  start = new (struct tm);
  end = new (struct tm);
  cur = NDomains;
  while (cur != NULL) {
    // Die jeweilige Domain Daten laden
    printf ("Lade die Ergebnisse von %s\n",cur->xmlChildrenNode->content);
    strptime ((char *)xmlGetProp(cur, (xmlChar *)"startzeit"),"%c",start);
    strptime ((char *)xmlGetProp(cur, (xmlChar *)"endzeit"),"%c",end); 
    // Die Domain Daten schreiben
    aktdomain = results->AddDomain ((char *)cur->xmlChildrenNode->content,
      atol((char *)xmlGetProp(cur,(xmlChar *)"summessages")),
      atol((char *)xmlGetProp(cur,(xmlChar *)"incoming")),
      atol((char *)xmlGetProp(cur,(xmlChar *)"outgoing")),
      atol((char *)xmlGetProp(cur,(xmlChar *)"incbytes")),
      atol((char *)xmlGetProp(cur,(xmlChar *)"outbytes")),
      atol((char *)xmlGetProp(cur,(xmlChar *)"local")),
      atol((char *)xmlGetProp(cur,(xmlChar *)"localbytes")),
      0,
      atol((char *)xmlGetProp(cur,(xmlChar *)"sumbytes")),
      atol((char *)xmlGetProp(cur,(xmlChar *)"sec")),
      start,
      end,
      usage,
      bytes);      
    // Dann die Member Daten laden
    member = cur->xmlChildrenNode->next;
    while (member != NULL) {
      // Hier die member laden
      if ((member->xmlChildrenNode != NULL) && (!xmlStrcmp(member->name,(xmlChar *)"MEMBER"))) {
        strptime ((char *)xmlGetProp(member, (xmlChar *)"startzeit"),"%c",start);
        strptime ((char *)xmlGetProp(member, (xmlChar *)"endzeit"),"%c",end);
        results->AddMember (aktdomain,
	  (char *)member->xmlChildrenNode->content,
          atol((char *)xmlGetProp(member,(xmlChar *)"summessages")),
          atol((char *)xmlGetProp(member,(xmlChar *)"incoming")),
          atol((char *)xmlGetProp(member,(xmlChar *)"outgoing")),
          atol((char *)xmlGetProp(member,(xmlChar *)"incbytes")),
          atol((char *)xmlGetProp(member,(xmlChar *)"outbytes")),
          atol((char *)xmlGetProp(member,(xmlChar *)"local")),
          atol((char *)xmlGetProp(member,(xmlChar *)"localbytes")),
          0,
          atol((char *)xmlGetProp(member,(xmlChar *)"sumbytes")),
          atol((char *)xmlGetProp(member,(xmlChar *)"sec")),
          start,
          end);
      }
      else if ((member->xmlChildrenNode != NULL) && (!xmlStrcmp(member->name,(xmlChar *)"YEAR"))) {
        // Hier werden dann die Monats Statistiken eingelesen
	// Also ins nchste Sub - Level absteigen
	month = member->xmlChildrenNode->next;
	year = atoi((char *)member->xmlChildrenNode->content);
	while (month != NULL) {
 	  if ((month->xmlChildrenNode != NULL) && (!xmlStrcmp(month->name,(xmlChar *)"MONTH"))) {
	    // Hier nun die Monats Daten einlesen
	    usage[atoi((char *)month->xmlChildrenNode->content)] = atol((char *)xmlGetProp(month,(xmlChar *)"mails"));
	    bytes[atoi((char *)month->xmlChildrenNode->content)] = atol((char *)xmlGetProp(month,(xmlChar *)"bytes"));
	  }
	  month = month->next;
	}
      }
      member = member->next;
    }
    // Hier die nun berechneten usage ergebnisse sichern
    results->ChangeUsage (aktdomain,usage,bytes);  
    cur = cur->next;
  }
  free (start);
  free (end);
}

// Ldt die bereits geparsten Files
void XMLInput::LoadFiles ()
{ xmlNodePtr cur;
  LogFiles *nelement,*aktelement;
  
  cur = NFiles;
  aktelement = NULL;
  nelement = NULL;
  while (cur != NULL) {
    // Die bereits geparsten Files einlesen
    nelement = new (LogFiles);
    nelement->next = NULL;
    nelement->prev = NULL;
    nelement->filename = (char *)malloc(strlen((char *)cur->xmlChildrenNode->content)+1);
    strcpy (nelement->filename,(char *)cur->xmlChildrenNode->content);
    nelement->processed = true;
    if (aktelement == NULL) {
      aktelement = nelement;
      lWurzel = nelement;
    }
    else {
      aktelement->next = nelement;
      nelement->prev = aktelement;
      aktelement = nelement;
    }
    cur = cur->next;
  }
}

// Ldt die noch nicht abgeschlossenen Nachrichten
void XMLInput::LoadMessages ()
{ xmlNodePtr cur;
  xmlNodePtr d;
  unsigned long msgnr,bytes,dnr;
  struct tm *datum;
  char *local=(char *)malloc(10);
  
  datum = new(struct tm);
  cur = NMessages;
  while (cur != NULL) {
    // Die noch offenen Nachrichten laden
    msgnr = atol ((char *)cur->xmlChildrenNode->content);
//    getdate_r (xmlGetProp(cur,(xmlChar *)"startzeit"),datum);
    strptime ((char *)xmlGetProp(cur, (xmlChar *)"startzeit"),"%c",datum);
    mhandler->newmessage (msgnr,datum);
    // Die Daten fr die info message laden
    bytes = atol ((char *)xmlGetProp(cur,(xmlChar *)"bytes"));
    mhandler->infomessage (msgnr,bytes,(char *)xmlGetProp(cur,(xmlChar *)"sender"),datum);
    // Dann die Deliveries laden
    d = cur->xmlChildrenNode->next->xmlChildrenNode->next;
    while (d != NULL) {
      // Die Deliveries laden
      dnr = atol ((char *)d->xmlChildrenNode->content);
      strptime ((char *)xmlGetProp(d, (xmlChar *)"zeitpunkt"),"%c",datum);
      if (strcmp((char *)xmlGetProp(d,(xmlChar *)"local"),"false")==0)
        strcpy (local,"local");
      else
        strcpy(local,"remote");
      mhandler->newdeliverie (dnr,msgnr,(char *)xmlGetProp(d,(xmlChar *)"recipient"),local,datum);
      mhandler->deliveriestate (dnr,(char *)xmlGetProp(d,(xmlChar *)"state"),(char *)xmlGetProp(d,(xmlChar *)"smessage"),datum);
      d = d->next;
    }  
    cur = cur->next;
  }
  free (datum);
  free (local);
}
